/*
 * Copyright (c) 2011 J. Lewis Muir <jlmuir@imca.aps.anl.gov>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

package imcacat.jaccordion;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.Iterator;

import javax.swing.JLabel;
import javax.swing.SwingUtilities;

import org.junit.Test;

/**
 * I test <code>JAccordion</code>.
 */
public class JAccordionTest {
  /* Defaults. */
  @Test
  public void testConstructor01() throws Exception {
    SwingUtilities.invokeAndWait(new Runnable() {
      @Override
      public void run() {
        JAccordion accordion = new JAccordion();
        assertEquals(0, accordion.sectionCount());
        assertFalse(accordion.isMaxOneSectionOpenEnabled());
      }
    });
  }

  /* Add panel from title and body. */
  @Test
  public void testAddSection01() throws Exception {
    SwingUtilities.invokeAndWait(new Runnable() {
      @Override
      public void run() {
        JAccordion accordion = new JAccordion();
        accordion.addSection("Characters", new JLabel("Luke Skywalker"));
        assertEquals(1, accordion.sectionCount());
        accordion.addSection("Locations", new JLabel("Tatooine"));
        assertEquals(2, accordion.sectionCount());
        accordion.addSection("Ships", new JLabel("TIE Fighter"));
        assertEquals(3, accordion.sectionCount());
      }
    });
  }

  /* Add panel. */
  @Test
  public void testAddSection02() throws Exception {
    SwingUtilities.invokeAndWait(new Runnable() {
      @Override
      public void run() {
        JAccordion accordion = new JAccordion();
        accordion.addSection(new JClosablePanel("Characters", new JLabel("Luke Skywalker")));
        assertEquals(1, accordion.sectionCount());
        accordion.addSection(new JClosablePanel("Locations", new JLabel("Tatooine")));
        assertEquals(2, accordion.sectionCount());
        accordion.addSection(new JClosablePanel("Ships", new JLabel("TIE Fighter")));
        assertEquals(3, accordion.sectionCount());
      }
    });
  }

  /* Single remove by index. */
  @Test
  public void testRemoveSection01() throws Exception {
    SwingUtilities.invokeAndWait(new Runnable() {
      @Override
      public void run() {
        JAccordion accordion = new JAccordion();
        JClosablePanel p = new JClosablePanel("Characters", new JLabel("Luke Skywalker"));
        accordion.addSection(p);
        accordion.removeSection(0);
        assertEquals(0, accordion.sectionCount());
      }
    });
  }

  /* Single remove by reference. */
  @Test
  public void testRemoveSection02() throws Exception {
    SwingUtilities.invokeAndWait(new Runnable() {
      @Override
      public void run() {
        JAccordion accordion = new JAccordion();
        JClosablePanel p = new JClosablePanel("Characters", new JLabel("Luke Skywalker"));
        accordion.addSection(p);
        accordion.removeSection(p);
        assertEquals(0, accordion.sectionCount());
      }
    });
  }

  /* Remove from top, middle, and bottom by index. */
  @Test
  public void testRemoveSection03() throws Exception {
    SwingUtilities.invokeAndWait(new Runnable() {
      @Override
      public void run() {
        JAccordion accordion = new JAccordion();
        JClosablePanel p1 = new JClosablePanel("Characters", new JLabel("Luke Skywalker"));
        accordion.addSection(p1);
        JClosablePanel p2 = new JClosablePanel("Locations", new JLabel("Tatooine"));
        accordion.addSection(p2);
        JClosablePanel p3 = new JClosablePanel("Ships", new JLabel("TIE Fighter"));
        accordion.addSection(p3);
        JClosablePanel p4 = new JClosablePanel("Creatures", new JLabel("Tauntaun"));
        accordion.addSection(p4);
        accordion.removeSection(0);
        accordion.removeSection(1);
        accordion.removeSection(1);
        accordion.removeSection(0);
        assertEquals(0, accordion.sectionCount());
      }
    });
  }

  /* Remove from top, middle, and bottom by reference. */
  @Test
  public void testRemoveSection04() throws Exception {
    SwingUtilities.invokeAndWait(new Runnable() {
      @Override
      public void run() {
        JAccordion accordion = new JAccordion();
        JClosablePanel p1 = new JClosablePanel("Characters", new JLabel("Luke Skywalker"));
        accordion.addSection(p1);
        JClosablePanel p2 = new JClosablePanel("Locations", new JLabel("Tatooine"));
        accordion.addSection(p2);
        JClosablePanel p3 = new JClosablePanel("Ships", new JLabel("TIE Fighter"));
        accordion.addSection(p3);
        JClosablePanel p4 = new JClosablePanel("Creatures", new JLabel("Tauntaun"));
        accordion.addSection(p4);
        accordion.removeSection(p1);
        accordion.removeSection(p3);
        accordion.removeSection(p4);
        accordion.removeSection(p2);
        assertEquals(0, accordion.sectionCount());
      }
    });
  }

  /* Index of -1 is illegal. */
  @Test
  public void testRemoveSection05() throws Exception {
    SwingUtilities.invokeAndWait(new Runnable() {
      @Override
      public void run() {
        JAccordion accordion = new JAccordion();
        JClosablePanel p1 = new JClosablePanel("Characters", new JLabel("Luke Skywalker"));
        accordion.addSection(p1);
        try {
          accordion.removeSection(-1);
          fail();
        } catch (IndexOutOfBoundsException e) {
        }
      }
    });
  }

  /* Insert at top and middle. */
  @Test
  public void testInsertSection01() throws Exception {
    SwingUtilities.invokeAndWait(new Runnable() {
      @Override
      public void run() {
        JAccordion accordion = new JAccordion();
        JClosablePanel p1 = new JClosablePanel("Characters", new JLabel("Luke Skywalker"));
        accordion.insertSection(p1, 0);
        JClosablePanel p2 = new JClosablePanel("Locations", new JLabel("Tatooine"));
        accordion.insertSection(p2, 0);
        JClosablePanel p3 = new JClosablePanel("Ships", new JLabel("TIE Fighter"));
        accordion.insertSection(p3, 1);
        assertEquals(3, accordion.sectionCount());
        assertEquals(p2, accordion.getSection(0));
        assertEquals(p3, accordion.getSection(1));
        assertEquals(p1, accordion.getSection(2));
        assertEquals(3, accordion.sectionCount());
      }
    });
  }

  /* Insert at top. */
  @Test
  public void testInsertSection02() throws Exception {
    SwingUtilities.invokeAndWait(new Runnable() {
      @Override
      public void run() {
        JAccordion accordion = new JAccordion();
        JClosablePanel p1 = new JClosablePanel("Characters", new JLabel("Luke Skywalker"));
        accordion.insertSection(p1, 0);
        JClosablePanel p2 = new JClosablePanel("Locations", new JLabel("Tatooine"));
        accordion.insertSection(p2, 0);
        JClosablePanel p3 = new JClosablePanel("Ships", new JLabel("TIE Fighter"));
        accordion.insertSection(p3, 0);
        assertEquals(3, accordion.sectionCount());
        assertEquals(p3, accordion.getSection(0));
        assertEquals(p2, accordion.getSection(1));
        assertEquals(p1, accordion.getSection(2));
        assertEquals(3, accordion.sectionCount());
      }
    });
  }

  /* Insert at bottom. */
  @Test
  public void testInsertSection03() throws Exception {
    SwingUtilities.invokeAndWait(new Runnable() {
      @Override
      public void run() {
        JAccordion accordion = new JAccordion();
        JClosablePanel p1 = new JClosablePanel("Characters", new JLabel("Luke Skywalker"));
        accordion.insertSection(p1, 0);
        JClosablePanel p2 = new JClosablePanel("Locations", new JLabel("Tatooine"));
        accordion.insertSection(p2, 1);
        JClosablePanel p3 = new JClosablePanel("Ships", new JLabel("TIE Fighter"));
        accordion.insertSection(p3, 2);
        assertEquals(3, accordion.sectionCount());
        assertEquals(p1, accordion.getSection(0));
        assertEquals(p2, accordion.getSection(1));
        assertEquals(p3, accordion.getSection(2));
        assertEquals(3, accordion.sectionCount());
      }
    });
  }

  /* Index of -1 is illegal. */
  @Test
  public void testInsertSection04() throws Exception {
    SwingUtilities.invokeAndWait(new Runnable() {
      @Override
      public void run() {
        JAccordion accordion = new JAccordion();
        JClosablePanel p1 = new JClosablePanel("Characters", new JLabel("Luke Skywalker"));
        try {
          accordion.insertSection(p1, -1);
          fail();
        } catch (IndexOutOfBoundsException e) {
        }
      }
    });
  }

  /* Basic. */
  @Test
  public void testGetSection01() throws Exception {
    SwingUtilities.invokeAndWait(new Runnable() {
      @Override
      public void run() {
        JAccordion accordion = new JAccordion();
        JClosablePanel p1 = new JClosablePanel("Characters", new JLabel("Luke Skywalker"));
        accordion.addSection(p1);
        JClosablePanel p2 = new JClosablePanel("Locations", new JLabel("Tatooine"));
        accordion.addSection(p2);
        JClosablePanel p3 = new JClosablePanel("Ships", new JLabel("TIE Fighter"));
        accordion.addSection(p3);
        assertEquals(p3, accordion.getSection(2));
        assertEquals(p2, accordion.getSection(1));
        assertEquals(p1, accordion.getSection(0));
      }
    });
  }

  /* Basic. */
  @Test
  public void testGetSections01() throws Exception {
    SwingUtilities.invokeAndWait(new Runnable() {
      @Override
      public void run() {
        JAccordion accordion = new JAccordion();
        JClosablePanel p1 = new JClosablePanel("Characters", new JLabel("Luke Skywalker"));
        accordion.addSection(p1);
        JClosablePanel p2 = new JClosablePanel("Locations", new JLabel("Tatooine"));
        accordion.addSection(p2);
        JClosablePanel p3 = new JClosablePanel("Ships", new JLabel("TIE Fighter"));
        accordion.addSection(p3);

        Iterator<JClosablePanel> it = accordion.getSections();
        JClosablePanel[] sections = { p1, p2, p3 };
        for (int i = 0; i < sections.length; i++) {
          assertEquals("at i=" + i, sections[i], it.next());
        }
      }
    });
  }

  /* Empty. */
  @Test
  public void testGetSections02() throws Exception {
    SwingUtilities.invokeAndWait(new Runnable() {
      @Override
      public void run() {
        JAccordion accordion = new JAccordion();
        assertFalse(accordion.getSections().hasNext());
      }
    });
  }

  /* Basic. */
  @Test
  public void testFindSection01() throws Exception {
    SwingUtilities.invokeAndWait(new Runnable() {
      @Override
      public void run() {
        JAccordion accordion = new JAccordion();
        JClosablePanel p1 = new JClosablePanel("Characters", new JLabel("Luke Skywalker"));
        accordion.addSection(p1);
        JClosablePanel p2 = new JClosablePanel("Locations", new JLabel("Tatooine"));
        accordion.addSection(p2);
        JClosablePanel p3 = new JClosablePanel("Ships", new JLabel("TIE Fighter"));
        accordion.addSection(p3);
        assertEquals(1, accordion.findSection(p2));
        assertEquals(0, accordion.findSection(p1));
        assertEquals(2, accordion.findSection(p3));
      }
    });
  }

  /* Nonexistent. */
  @Test
  public void testFindSection02() throws Exception {
    SwingUtilities.invokeAndWait(new Runnable() {
      @Override
      public void run() {
        JAccordion accordion = new JAccordion();
        JClosablePanel p1 = new JClosablePanel("Characters", new JLabel("Luke Skywalker"));
        assertEquals(-1, accordion.findSection(p1));
        accordion.addSection(p1);
        JClosablePanel p2 = new JClosablePanel("Locations", new JLabel("Tatooine"));
        assertEquals(-1, accordion.findSection(p2));
      }
    });
  }

  /* Basic. */
  @Test
  public void testFindSectionByTitle01() throws Exception {
    SwingUtilities.invokeAndWait(new Runnable() {
      @Override
      public void run() {
        JAccordion accordion = new JAccordion();
        JClosablePanel p1 = new JClosablePanel("Characters", new JLabel("Luke Skywalker"));
        accordion.addSection(p1);
        JClosablePanel p2 = new JClosablePanel("Locations", new JLabel("Tatooine"));
        accordion.addSection(p2);
        JClosablePanel p3 = new JClosablePanel("Ships", new JLabel("TIE Fighter"));
        accordion.addSection(p3);
        assertEquals(0, accordion.findSectionByTitle("Characters"));
        assertEquals(1, accordion.findSectionByTitle("Locations"));
        assertEquals(2, accordion.findSectionByTitle("Ships"));
      }
    });
  }

  /* Nonexistent. */
  @Test
  public void testFindSectionByTitle02() throws Exception {
    SwingUtilities.invokeAndWait(new Runnable() {
      @Override
      public void run() {
        JAccordion accordion = new JAccordion();
        JClosablePanel p1 = new JClosablePanel("Characters", new JLabel("Luke Skywalker"));
        assertEquals(-1, accordion.findSectionByTitle("Characters"));
        accordion.addSection(p1);
        assertEquals(-1, accordion.findSectionByTitle("Locations"));
      }
    });
  }

  /* Finds first occurrence when more than one section with same title. */
  @Test
  public void testFindSectionByTitle03() throws Exception {
    SwingUtilities.invokeAndWait(new Runnable() {
      @Override
      public void run() {
        JAccordion accordion = new JAccordion();
        JClosablePanel p1 = new JClosablePanel("Characters", new JLabel("Luke Skywalker"));
        accordion.addSection(p1);
        JClosablePanel p2 = new JClosablePanel("Characters", new JLabel("Luke Skywalker"));
        accordion.addSection(p2);
        assertEquals(0, accordion.findSectionByTitle("Characters"));
        JClosablePanel p3 = new JClosablePanel("Locations", new JLabel("Tatooine"));
        accordion.addSection(p3);
        JClosablePanel p4 = new JClosablePanel("Locations", new JLabel("Tatooine"));
        accordion.addSection(p4);
        assertEquals(2, accordion.findSectionByTitle("Locations"));
      }
    });
  }

  /* Basic. */
  @Test
  public void testMaxOneSectionOpen01() throws Exception {
    SwingUtilities.invokeAndWait(new Runnable() {
      @Override
      public void run() {
        JAccordion accordion = new JAccordion();
        accordion.enableMaxOneSectionOpen();
        assertTrue(accordion.isMaxOneSectionOpenEnabled());
        accordion.disableMaxOneSectionOpen();
        assertFalse(accordion.isMaxOneSectionOpenEnabled());
        accordion.setMaxOneSectionOpenEnabled(true);
        assertTrue(accordion.isMaxOneSectionOpenEnabled());
        accordion.setMaxOneSectionOpenEnabled(false);
        assertFalse(accordion.isMaxOneSectionOpenEnabled());
      }
    });
  }

  /* Disabled. */
  @Test
  public void testMaxOneSectionOpen02() throws Exception {
    SwingUtilities.invokeAndWait(new Runnable() {
      @Override
      public void run() {
        JAccordion accordion = new JAccordion();
        JClosablePanel p1 = new JClosablePanel("Characters", new JLabel("Luke Skywalker"));
        accordion.addSection(p1);
        JClosablePanel p2 = new JClosablePanel("Locations", new JLabel("Tatooine"));
        accordion.addSection(p2);
        p1.makeOpen();
        p2.makeOpen();
        assertTrue(p1.isOpen());
        assertTrue(p2.isOpen());
      }
    });
  }

  /* Enabled. */
  @Test
  public void testMaxOneSectionOpen03() throws Exception {
    SwingUtilities.invokeAndWait(new Runnable() {
      @Override
      public void run() {
        JAccordion accordion = new JAccordion();
        JClosablePanel p1 = new JClosablePanel("Characters", new JLabel("Luke Skywalker"));
        accordion.addSection(p1);
        JClosablePanel p2 = new JClosablePanel("Locations", new JLabel("Tatooine"));
        accordion.addSection(p2);
        JClosablePanel p3 = new JClosablePanel("Ships", new JLabel("TIE Fighter"));
        accordion.addSection(p3);
        accordion.enableMaxOneSectionOpen();
        p1.makeOpen();
        p2.makeOpen();
        p3.makeOpen();
        assertFalse(p1.isOpen());
        assertFalse(p2.isOpen());
        assertTrue(p3.isOpen());
      }
    });
  }

  /* Enabled with sections already open. */
  @Test
  public void testMaxOneSectionOpen04() throws Exception {
    SwingUtilities.invokeAndWait(new Runnable() {
      @Override
      public void run() {
        JAccordion accordion = new JAccordion();
        JClosablePanel p1 = new JClosablePanel("Characters", new JLabel("Luke Skywalker"));
        accordion.addSection(p1);
        JClosablePanel p2 = new JClosablePanel("Locations", new JLabel("Tatooine"));
        accordion.addSection(p2);
        JClosablePanel p3 = new JClosablePanel("Ships", new JLabel("TIE Fighter"));
        accordion.addSection(p3);
        p1.makeOpen();
        p2.makeOpen();
        p3.makeOpen();
        assertTrue(p1.isOpen());
        assertTrue(p2.isOpen());
        assertTrue(p3.isOpen());
        accordion.enableMaxOneSectionOpen();
        assertTrue(p1.isOpen());
        assertFalse(p2.isOpen());
        assertFalse(p3.isOpen());
      }
    });
  }

  /* Enabled with just one section already open. */
  @Test
  public void testMaxOneSectionOpen05() throws Exception {
    SwingUtilities.invokeAndWait(new Runnable() {
      @Override
      public void run() {
        JAccordion accordion = new JAccordion();
        JClosablePanel p1 = new JClosablePanel("Characters", new JLabel("Luke Skywalker"));
        accordion.addSection(p1);
        JClosablePanel p2 = new JClosablePanel("Locations", new JLabel("Tatooine"));
        accordion.addSection(p2);
        JClosablePanel p3 = new JClosablePanel("Ships", new JLabel("TIE Fighter"));
        accordion.addSection(p3);
        p3.makeOpen();
        accordion.enableMaxOneSectionOpen();
        assertFalse(p1.isOpen());
        assertFalse(p2.isOpen());
        assertTrue(p3.isOpen());
      }
    });
  }

  /* Enabled with no section already open. */
  @Test
  public void testMaxOneSectionOpen06() throws Exception {
    SwingUtilities.invokeAndWait(new Runnable() {
      @Override
      public void run() {
        JAccordion accordion = new JAccordion();
        JClosablePanel p1 = new JClosablePanel("Characters", new JLabel("Luke Skywalker"));
        accordion.addSection(p1);
        JClosablePanel p2 = new JClosablePanel("Locations", new JLabel("Tatooine"));
        accordion.addSection(p2);
        JClosablePanel p3 = new JClosablePanel("Ships", new JLabel("TIE Fighter"));
        accordion.addSection(p3);
        accordion.enableMaxOneSectionOpen();
        assertFalse(p1.isOpen());
        assertFalse(p2.isOpen());
        assertFalse(p3.isOpen());
      }
    });
  }

  /* Enabled with one section already open and add an open section. */
  @Test
  public void testMaxOneSectionOpen07() throws Exception {
    SwingUtilities.invokeAndWait(new Runnable() {
      @Override
      public void run() {
        JAccordion accordion = new JAccordion();
        JClosablePanel p1 = new JClosablePanel("Characters", new JLabel("Luke Skywalker"));
        accordion.addSection(p1);
        p1.makeOpen();
        accordion.enableMaxOneSectionOpen();
        JClosablePanel p2 = new JClosablePanel("Locations", new JLabel("Tatooine"));
        p2.makeOpen();
        accordion.addSection(p2);
        assertFalse(p1.isOpen());
        assertTrue(p2.isOpen());
      }
    });
  }
}
