/*
 * Copyright (c) 2011 J. Lewis Muir <jlmuir@imca.aps.anl.gov>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

package imcacat.jaccordion;

import java.awt.Component;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.JPanel;
import javax.swing.Spring;
import javax.swing.SpringLayout;

/**
 * I am an <q>accordion</q> component providing a vertical list of titled
 * components which may be opened or closed.
 * <p>
 * Example usage:
 * <p>
 * 
 * <pre>
 * JAccordion accordion = new JAccordion();
 * accordion.addSection("Characters", new JLabel("Characters Panel Here"));
 * accordion.addSection("Locations", new JLabel("Locations Panel Here"));
 * accordion.addSection("Ships", new JLabel("Ships Panel Here"));
 * </pre>
 */
public class JAccordion extends JPanel {
  private static final long serialVersionUID = 1L;

  private static final int MARGIN_PADDING = 3;
  private static final int SECTION_PADDING = 1;

  private final SpringLayout layout = new SpringLayout();
  private final ArrayList<SectionItem> items = new ArrayList<SectionItem>();
  private boolean maxOneSectionOpenEnabled = false;

  public JAccordion() {
    super();
    setLayout(layout);
  }

  public int sectionCount() {
    return items.size();
  }

  public Iterator<JClosablePanel> getSections() {
    ArrayList<JClosablePanel> result = new ArrayList<JClosablePanel>(items.size());
    for (SectionItem each : items)
      result.add(each.getSection());
    return result.iterator();
  }

  public int findSection(JClosablePanel section) {
    int size = items.size();
    for (int i = 0; i < size; i++) {
      if (items.get(i).getSection().equals(section)) return i;
    }
    return -1;
  }

  public int findSectionByTitle(String title) {
    int size = items.size();
    for (int i = 0; i < size; i++) {
      if (items.get(i).getSection().getTitle().equals(title)) return i;
    }
    return -1;
  }

  public JClosablePanel getSection(int index) {
    return items.get(index).getSection();
  }

  public void addSection(String title, Component component) {
    addSection(new JClosablePanel(title, component));
  }

  public void addSection(JClosablePanel section) {
    insertSection(section, items.size());
  }

  public void insertSection(String title, Component component, int index) {
    insertSection(new JClosablePanel(title, component), index);
  }

  public void insertSection(JClosablePanel section, int index) {
    int componentsIndex = -1;
    if (index != items.size() && !items.isEmpty()) {
      componentsIndex = findComponent(items.get(index).getSection());
    }
    addSectionItem(section, index);
    add(section, componentsIndex);
    int north = index - 1;
    int south = index + 1;
    setConstraints(section,
      (north >= 0 && north < items.size()) ? items.get(north).getSection() : null,
      (south >= 0 && south < items.size()) ? items.get(south).getSection() : null);
    validate();
  }

  public void removeSection(JClosablePanel section) {
    int index = findSection(section);
    if (index < 0) return;
    removeSection(index);
  }

  public void removeSection(int index) {
    remove(items.get(index).getSection());
    int north = index - 1;
    int subject = index + 1;
    int south = index + 2;
    if (index == items.size() - 1) {
      north--;
      subject = north + 1;
      south = north + 2;
    }
    setConstraints(
      (subject >= 0 && subject < items.size()) ? items.get(subject).getSection() : null,
      (north >= 0 && north < items.size()) ? items.get(north).getSection() : null,
      (south >= 0 && south < items.size()) ? items.get(south).getSection() : null);
    validate();
    repaint();
    removeSectionItem(index);
  }

  public boolean isMaxOneSectionOpenEnabled() {
    return maxOneSectionOpenEnabled;
  }

  public void setMaxOneSectionOpenEnabled(boolean enabled) {
    maxOneSectionOpenEnabled = enabled;
    if (!enabled) return;
    JClosablePanel section = getFirstOpenSection();
    if (section == null) return;
    closeOtherSections(section);
  }

  public void enableMaxOneSectionOpen() {
    setMaxOneSectionOpenEnabled(true);
  }

  public void disableMaxOneSectionOpen() {
    setMaxOneSectionOpenEnabled(false);
  }

  private void setConstraints(JClosablePanel subject, JClosablePanel north, JClosablePanel south) {
    if (subject == null) return;

    JClosablePanel[] all = { subject, north, south };
    for (JClosablePanel each : all) {
      if (each == null) continue;
      layout.putConstraint(SpringLayout.WEST, each, MARGIN_PADDING, SpringLayout.WEST, this);
    }

    if (north == null) {
      layout.putConstraint(SpringLayout.NORTH, subject, MARGIN_PADDING, SpringLayout.NORTH, this);
      SpringLayout.Constraints subjectConstraints = layout.getConstraints(subject);
      subjectConstraints.setConstraint(SpringLayout.EAST, Spring.width(subject));
      layout.putConstraint(SpringLayout.EAST, this, MARGIN_PADDING, SpringLayout.EAST, subject);
    } else {
      layout.putConstraint(SpringLayout.NORTH, subject, SECTION_PADDING, SpringLayout.SOUTH, north);
      layout.putConstraint(SpringLayout.EAST, subject, -MARGIN_PADDING, SpringLayout.EAST, this);
    }

    if (south == null) {
      layout.putConstraint(SpringLayout.SOUTH, this,
        Spring.constant(MARGIN_PADDING, MARGIN_PADDING, Integer.MAX_VALUE), SpringLayout.SOUTH,
        subject);
    } else {
      layout.putConstraint(SpringLayout.NORTH, south, SECTION_PADDING, SpringLayout.SOUTH, subject);
      layout.putConstraint(SpringLayout.EAST, south, -MARGIN_PADDING, SpringLayout.EAST, this);
    }
  }

  private void addSectionItem(final JClosablePanel section, int index) {
    OpenedListener listener = new OpenedListener() {
      @Override
      public void opened(JClosablePanel panel) {
        sectionOpened(section);
      }
    };
    section.addOpenedListener(listener);
    items.add(index, new SectionItem(section, listener));
    if (section.isOpen()) sectionOpened(section);
  }

  private void removeSectionItem(int index) {
    SectionItem item = items.get(index);
    item.getSection().removeOpenedListener(item.getListener());
    items.remove(index);
  }

  private int findComponent(Component needle) {
    if (needle == null) return -1;
    Component[] haystack = getComponents();
    for (int i = 0; i < haystack.length; i++) {
      if (needle.equals(haystack[i])) return i;
    }
    return -1;
  }

  private JClosablePanel getFirstOpenSection() {
    for (SectionItem each : items) {
      if (each.getSection().isOpen()) return each.getSection();
    }
    return null;
  }

  private void sectionOpened(JClosablePanel p) {
    if (p == null) return;
    if (!maxOneSectionOpenEnabled) return;
    closeOtherSections(p);
  }

  private void closeOtherSections(JClosablePanel p) {
    for (SectionItem each : items) {
      if (!each.getSection().equals(p)) each.getSection().makeClosed();
    }
  }
}
